"""
Copyright (c) Huawei Technologies Co., Ltd. 2018-2020. All rights reserved.
"""

from aoc import NcsService, devicemgr
from aoc import datastore
from aoc.exception.aocexceptions import AOCException


class L3VPNService(NcsService):
    """Convert L3VPN XML files into NETCONF packets"""
    def ncs_map(self, request, aoccontext=None, template=None):
        """
        Entry point function
        :param request: XML packet
        :param aoc_context: dict, contains
                properties['transactionId']
                properties['deviceType']
                properties['deviceVendor']
                properties['deviceVersion']
        :param template: Jinja2 template
        :return: opf.models.mapping_pb2.NcsServiceOutput
        """
        l3vpn_dict = self.xmltodict(request.xml)

        # Check whether the main interface exists.
        self.checkInterfaceIsExist(request, aoccontext)

        # Update the interface description.
        self.updateDes(l3vpn_dict)

        self.logger.info(str(l3vpn_dict))
        res = self.render('l3vpn/servicepoint.j2', l3vpn_dict)
        self.logger.info(res)
        return res

    def updateDes(self, l3vpn_dict):
        """
        Update the interface description.
        :param l3vpn_dict Dict data of L3VPN
        :return: NONE
        """
        if '3G' in l3vpn_dict['l3vpn']['vrfName']:
            l3vpn_dict.update({"vrfDes": '3G'})
            l3vpn_dict['l3vpn'].update({"description": '3G'})

    def checkInterfaceIsExist(self, request, aoccontext):
        """
        Check whether the main interface to which the sub-interface belongs exists.
        :param request: XML packet
        :param aoc_context: dict, contains
               properties['transactionId']
               properties['deviceType']
               properties['deviceVendor']
               properties['deviceVersion']
        :return: NONE
        """
        device_id = devicemgr.query_neid(request.xmldictnode.l3vpn.deviceName)

        # GigabitEthernet3/0/0.801 ,  get GigabitEthernet3/0/0
        subIfName = request.xmldictnode.l3vpn.ifName
        pointIndex = subIfName.find('.')
        parentName = subIfName[0:pointIndex]
        self.logger.info('deviceId:%s, parentName:%s' % (device_id, parentName))
        if devicemgr.is_snd(device_id, 'NE8000M8SPC300_SND') or devicemgr.is_snd(device_id, 'NE8000_M8'):
            path = "huawei-ac-nes:inventory-cfg/nes/ne/" + device_id + "/huawei-ifm:ifm/interfaces/"
            output = datastore.read_datastore_rdb(aoccontext, path)
            asnindex = str(output).find(parentName)
            if asnindex == -1:
                raise AOCException(parentName + ' not exist in device!')

    def add_filters(self):
        """
        override add_filters add user define filters
        Define a filter, which is invoked in Jinja2.
        """
        result = {"is_snd": self.is_snd}
        return result

    def is_snd(self, nename, sndid):
        """
        Determine whether the package is an SND package (provided to the filter for use).
        """
        neid = devicemgr.query_neid(nename)
        return devicemgr.is_snd(neid, sndid)
